# pylint: disable=no-member, unused-import
"""Serveur minimal de démo

export FLASK_APP=server.py FLASK_ENV=development; flask run
"""

import sqlite3
from flask import Flask, render_template, redirect, url_for, abort, request, g

# les fichiers du dossier static semblent vus de la racine
app = Flask(__name__, static_url_path="")
app.jinja_env.globals["title"] = "Mon application Flask"
app.jinja_env.globals["website"] = "https://romulusfr.github.io/unc-s2-web-ihm/"

# attention : ceci NE MARCHE PAS **UTILISER UN SERVICE EXTERNE COMME UNE BD**
# db_users = [("demo@domain.com", "utilisateur de démonstration")]

#  https://docs.python.org/3/library/sqlite3.htm
DATABASE = "db.sqlite"


def get_db():
    """renvoie une connection à la BD"""
    db = getattr(g, "_database", None)
    if db is None:
        db = g._database = sqlite3.connect(DATABASE)
        # disable automcommit
        db.isolation_level = None
        db.row_factory = sqlite3.Row
    return db


@app.teardown_appcontext
def close_connection(_exception):
    """ferme correctement la connection"""
    db = getattr(g, "_database", None)
    if db is not None:
        db.close()


def user_add(email, name):
    """ajoute un utilisateur"""
    cur = get_db().cursor()
    result = []
    try:
        cur.execute("insert into user VALUES(?, ?);", (email, name))
        result = cur.fetchall()
        app.logger.info(result)
        return None
    except sqlite3.Error as err:
        app.logger.fatal(err)
        # raise RuntimeError from err
        return err


def users_get():
    """renvoie la liste des utilisateurs"""
    cur = get_db().cursor()
    cur.execute("select * from user;")
    results = cur.fetchall()
    app.logger.info(results)
    return results


@app.route("/")
def index():
    """Page d'accueil"""
    return render_template("index.jinja2.html")


@app.route("/register")
def register():
    """Formulaire d'enregistrement"""
    return render_template("register.jinja2.html")


@app.route("/submit", methods=["GET", "POST"])
def submit():
    """Enregistrement"""
    if request.method == "POST":
        app.logger.info(request.form)
        error = user_add(request.form["email"], request.form["name"])
        if error:
            return render_template("error.jinja2.html", error=error)
    return redirect(url_for("index"))


@app.route("/status/<status_code>")
def status(status_code):
    """route d'erreur pour démo/tests"""
    app.logger.warning(status_code)
    return abort(int(status_code))


@app.route("/users/")
def users():
    """Tous les utilisateurs"""
    return render_template("users.jinja2.html", users=users_get())


# https://flask.palletsp#rojects.com/en/2.0.x/quickstart/
# $env:FLASK_APP = "server_with_db"; $env:FLASK_ENV = "development"; py -m flask run
# https://flask.palletsprojects.com/en/2.0.x/quickstart/#rendering-templates


def init_db():
    """initialisation de la BD"""
    with app.app_context():
        try:
            cur = get_db().cursor()
            cur.execute("select sqlite_version();")
            app.logger.debug(cur.fetchone())
            cur.execute(
                "CREATE TABLE IF NOT EXISTS user(email text primary key, name text not null);"
            )
            # cur.execute("INSERT INTO user VALUES('demo@domain.com', 'utilisateur de démonstration')")
        except sqlite3.Error as err:
            app.logger.fatal(err)


if __name__ == "__main__":
    app.run(debug=True)
    # introspection : la liste des routes déclarées
    for rule in app.url_map.iter_rules():
        app.logger.debug(rule)
    init_db()
